use anyhow::Result;
use async_graphql::SimpleObject;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool, Type};

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize, Type)]
pub struct Typ {
    pub typ_id: i32,

    /// Name eines Typs
    pub name: String,
}

impl Typ {
    pub async fn create(pool: &PgPool, name: &str) -> Result<Typ> {
        let row = sqlx::query!("INSERT INTO typen(name) VALUES ($1) RETURNING typ_id", name)
            .fetch_one(pool)
            .await?;

        Ok(Typ {
            typ_id: row.typ_id,
            name: name.to_string(),
        })
    }

    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Typ> {
        let row = sqlx::query_as!(Typ, "SELECT * FROM typen WHERE typ_id = $1", id)
            .fetch_one(pool)
            .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Typ>> {
        let rows = sqlx::query_as!(Typ, "SELECT typ_id, name FROM typen")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn update(pool: &PgPool, id: &i32, name: &str) -> Result<Typ> {
        sqlx::query!("UPDATE typen SET name=$1 WHERE typ_id = $2", name, id)
            .execute(pool)
            .await?;

        let t = Typ::read_one(pool, id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<()> {
        sqlx::query!("DELETE FROM typen WHERE typ_id = $1", id)
            .execute(pool)
            .await?;

        Ok(())
    }
}
